@@ -1,4 +1,4 @@
-/*      $Id: compositor.c,v 1.62 2004/12/12 11:53:55 olivier Exp $
+/*      $Id: compositor.c,v 1.69 2004/12/16 23:02:53 olivier Exp $
 
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -38,6 +38,7 @@
 #include "client.h"
 #include "frame.h"
 #include "hints.h"
+#include "compositor.h"
 
 #ifdef HAVE_COMPOSITOR
 
@@ -1481,6 +1482,7 @@ add_win (DisplayInfo *display_info, Window id, Client *c, guint opacity)
     {
         map_win (new);
     }
+    TRACE ("window 0x%lx added", id);
 
     myDisplayUngrabServer (display_info);
 }
@@ -1489,9 +1491,9 @@ void
 restack_win (CWindow *cw, Window above)
 {
     ScreenInfo *screen_info = NULL;
+    Window previous_above = None;
     GList *sibling;
     GList *next;
-    GList *index;
 
     g_return_if_fail (cw != NULL);
     TRACE ("entering restack_win");
@@ -1499,30 +1501,44 @@ restack_win (CWindow *cw, Window above)
     screen_info = cw->screen_info;
     sibling = g_list_find (screen_info->cwindows, (gconstpointer) cw);
     next = g_list_next (sibling);
-    screen_info->cwindows = g_list_delete_link (screen_info->cwindows, sibling);
 
-    for (index = screen_info->cwindows; index; index = g_list_next (index))
+    if (next)
     {
-        CWindow *cw2 = (CWindow *) index->data;
-        if (cw2->id == above)
+        CWindow *ncw = (CWindow *) next;
+        previous_above = ncw->id;
+    }
+
+    if (previous_above != above)
+    {
+        GList *index;
+
+        for (index = screen_info->cwindows; index; index = g_list_next (index))
         {
-            break;
+            CWindow *cw2 = (CWindow *) index->data;
+            if (cw2->id == above)
+            {
+                break;
+            }
         }
-    }
 
-    if (index != NULL)
-    {
-        screen_info->cwindows =  g_list_insert_before (screen_info->cwindows, index, cw);
-    }
-    else if (above == None)
-    {
-        /* Insert at bottom of window stack */
-        screen_info->cwindows =  g_list_append (screen_info->cwindows, cw);
-    }
-    else
-    {
-        /* Insert at top of window stack */
-        screen_info->cwindows =  g_list_prepend (screen_info->cwindows, cw);
+        if (index != NULL)
+        {
+            screen_info->cwindows = g_list_delete_link (screen_info->cwindows, sibling);
+            screen_info->cwindows = g_list_insert_before (screen_info->cwindows, index, cw);
+        }
+        else if (above == None)
+        {
+            /* Insert at bottom of window stack */
+            screen_info->cwindows = g_list_delete_link (screen_info->cwindows, sibling);
+            screen_info->cwindows = g_list_append (screen_info->cwindows, cw);
+        }
+        else
+        {
+            /* Don't know what to do */
+            g_warning ("The window 0x%lx has not been restacked\n"
+                       "because the specified sibling 0x%lx was "
+                       "not found in our stack", cw->id, above);
+        }
     }
 }
 
@@ -1833,6 +1849,68 @@ compositorHandleCirculateNotify (DisplayInfo *display_info, XCirculateEvent *ev)
     add_repair (display_info);
 }
 
+static void
+compositorHandleCreateNotify (DisplayInfo *display_info, XCreateWindowEvent *ev)
+{
+    g_return_if_fail (display_info != NULL);
+    g_return_if_fail (ev != NULL);
+    TRACE ("entering compositorHandleCreateNotify for 0x%lx", ev->window);
+
+    if (!(display_info->enable_compositor))
+    {
+        TRACE ("compositor disabled");
+        return;
+    }
+    
+    /* 
+       We are only interested in top level windows, other will
+       be caught by the WM.
+     */
+    if (myDisplayGetScreenFromRoot (display_info, ev->parent) != NULL)
+    {
+        compositorAddWindow (display_info, ev->window, NULL);
+    }
+}
+
+static void
+compositorHandleReparentNotify (DisplayInfo *display_info, XReparentEvent *ev)
+{
+    g_return_if_fail (display_info != NULL);
+    g_return_if_fail (ev != NULL);
+    TRACE ("entering compositorHandleReparentNotify for 0x%lx", ev->window);
+
+    if (!(display_info->enable_compositor))
+    {
+        TRACE ("compositor disabled");
+        return;
+    }
+    
+    if (myDisplayGetScreenFromRoot (display_info, ev->parent) != NULL)
+    {
+        compositorAddWindow (display_info, ev->window, NULL);
+    }
+    else
+    {
+        compositorRemoveWindow (display_info, ev->window);
+    }
+}
+
+static void
+compositorHandleDestroyNotify (DisplayInfo *display_info, XDestroyWindowEvent *ev)
+{
+    g_return_if_fail (display_info != NULL);
+    g_return_if_fail (ev != NULL);
+    TRACE ("entering compositorHandleDestroyNotify for 0x%lx", ev->window);
+
+    if (!(display_info->enable_compositor))
+    {
+        TRACE ("compositor disabled");
+        return;
+    }
+    
+    compositorRemoveWindow (display_info, ev->window);
+}
+
 #endif /* HAVE_COMPOSITOR */
 
 void
@@ -1883,10 +1961,6 @@ compositorMapWindow (DisplayInfo *display_info, Window id)
             map_win (cw);
         }
     }
-    else
-    {
-        add_win (display_info, id, NULL, NET_WM_OPAQUE);
-    }
 #endif /* HAVE_COMPOSITOR */
 }
 
@@ -1933,7 +2007,14 @@ compositorAddWindow (DisplayInfo *display_info, Window id, Client *c)
     cw = find_cwindow_in_display (display_info, id);
     if (cw)
     {
-        cw->c = c;
+        /* 
+           The compositor window is already known, just update the client if
+           that is meaningfull...
+         */
+        if (c)
+        {
+            cw->c = c;
+        }
     }
     else
     {
@@ -1984,10 +2065,22 @@ compositorHandleEvent (DisplayInfo *display_info, XEvent *ev)
         return;
     }
 
-    if (ev->type == ConfigureNotify)
+    if (ev->type == CreateNotify)
+    {
+        compositorHandleCreateNotify (display_info, (XCreateWindowEvent *) ev);
+    }
+    else if (ev->type == DestroyNotify)
+    {
+        compositorHandleDestroyNotify (display_info, (XDestroyWindowEvent *) ev);
+    }
+    else if (ev->type == ConfigureNotify)
     {
         compositorHandleConfigureNotify (display_info, (XConfigureEvent *) ev);
     }
+    else if (ev->type == ReparentNotify)
+    {
+        compositorHandleReparentNotify (display_info, (XReparentEvent *) ev);
+    }
     else if (ev->type == Expose)
     {
         compositorHandleExpose (display_info, (XExposeEvent *) ev);
@@ -2043,7 +2136,7 @@ compositorInitDisplay (DisplayInfo *display_info)
                                 &display_info->composite_event_base,
                                 &display_info->composite_error_base))
     {
-        g_warning ("%s: The display does not support the XComposite extension.", PACKAGE);
+        g_warning ("The display does not support the XComposite extension.");
         display_info->have_composite = FALSE;
         display_info->composite_event_base = 0;
         display_info->composite_error_base = 0;
@@ -2056,7 +2149,7 @@ compositorInitDisplay (DisplayInfo *display_info)
                             &display_info->damage_event_base,
                             &display_info->damage_error_base))
     {
-        g_warning ("%s: The display does not support the XDamage extension.", PACKAGE);
+        g_warning ("The display does not support the XDamage extension.");
         display_info->have_damage = FALSE;
         display_info->damage_event_base = 0;
         display_info->damage_error_base = 0;
@@ -2069,7 +2162,7 @@ compositorInitDisplay (DisplayInfo *display_info)
                             &display_info->fixes_event_base,
                             &display_info->fixes_error_base))
     {
-        g_warning ("%s: The display does not support the XFixes extension.", PACKAGE);
+        g_warning ("The display does not support the XFixes extension.");
         display_info->have_fixes = FALSE;
         display_info->fixes_event_base = 0;
         display_info->fixes_error_base = 0;
@@ -2087,7 +2180,7 @@ compositorInitDisplay (DisplayInfo *display_info)
                                     && (display_info->have_fixes));
     if (!display_info->enable_compositor)
     {
-        g_warning ("%s: Compositing manager disabled.", PACKAGE);
+        g_warning ("Compositing manager disabled.");
     }
 #else /* HAVE_COMPOSITOR */
     display_info->enable_compositor = FALSE;
@@ -2141,7 +2234,7 @@ compositorManageScreen (ScreenInfo *screen_info, gboolean manual_redirect)
     XSync (display_info->dpy, FALSE);
     if (gdk_error_trap_pop ())
     {
-        g_warning ("%s: Another compositing manager is running on screen %i", PACKAGE, screen_info->screen);
+        g_warning ("Another compositing manager is running on screen %i", screen_info->screen);
         return FALSE;
     }
 
@@ -2156,7 +2249,7 @@ compositorManageScreen (ScreenInfo *screen_info, gboolean manual_redirect)
                                                             screen_info->screen));
     if (!visual_format)
     {
-        g_warning ("%s: Cannot find visual format on screen %i", PACKAGE, screen_info->screen);
+        g_warning ("Cannot find visual format on screen %i", screen_info->screen);
         return FALSE;
     }
 
@@ -2166,7 +2259,7 @@ compositorManageScreen (ScreenInfo *screen_info, gboolean manual_redirect)
 
     if (screen_info->rootPicture == None)
     {
-        g_warning ("%s: Cannot create root picture on screen %i", PACKAGE, screen_info->screen);
+        g_warning ("Cannot create root picture on screen %i", screen_info->screen);
         return FALSE;
     }
 
