@@ -1,4 +1,4 @@
-/*      $Id: events.c,v 1.293 2004/12/11 16:41:06 olivier Exp $
+/*      $Id: events.c,v 1.297 2004/12/19 21:23:58 olivier Exp $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -54,6 +54,10 @@
 #include "compositor.h"
 #include "events.h"
 
+#ifndef CHECK_BUTTON_TIME
+#define CHECK_BUTTON_TIME 0
+#endif
+
 #define WIN_IS_BUTTON(win)      ((win == MYWINDOW_XWINDOW(c->buttons[HIDE_BUTTON])) || \
                                  (win == MYWINDOW_XWINDOW(c->buttons[CLOSE_BUTTON])) || \
                                  (win == MYWINDOW_XWINDOW(c->buttons[MAXIMIZE_BUTTON])) || \
@@ -229,6 +233,7 @@ typeOfClick (ScreenInfo *screen_info, Window w, XEvent * ev, gboolean allow_doub
     return (XfwmButtonClickType) passdata.clicks;
 }
 
+#if CHECK_BUTTON_TIME
 static gboolean
 check_button_time (XButtonEvent *ev)
 {
@@ -242,7 +247,8 @@ check_button_time (XButtonEvent *ev)
     last_button_time = ev->time;
     return TRUE;
 }
-
+ #endif
+ 
 static void
 clear_timeout (void)
 {
@@ -317,7 +323,7 @@ spawn_shortcut (ScreenInfo *screen_info, int i)
     {
         if (error)
         {
-            g_warning ("%s: %s", g_get_prgname (), error->message);
+            g_warning ("%s", error->message);
             g_error_free (error);
         }
     }
@@ -827,13 +833,15 @@ handleButtonPress (DisplayInfo *display_info, XButtonEvent * ev)
 
     TRACE ("entering handleButtonPress");
 
+#if CHECK_BUTTON_TIME
     /* Avoid treating the same event twice */
     if (!check_button_time (ev))
     {
         TRACE ("ignoring ButtonPress event because it has been already handled");
         return;
     }
-
+#endif
+     
     c = myDisplayGetClientFromWindow (display_info, ev->window, ANY);
     if (c)
     {
@@ -1026,12 +1034,14 @@ handleButtonRelease (DisplayInfo *display_info, XButtonEvent * ev)
     ScreenInfo *screen_info = NULL;
     TRACE ("entering handleButtonRelease");
 
+#if CHECK_BUTTON_TIME
     /* Avoid treating the same event twice */
     if (!check_button_time (ev))
     {
         TRACE ("ignoring ButtonRelease event because it has been already handled");
         return;
     }
+#endif
 
     /* Get the screen structure from the root of the event */
     screen_info = myDisplayGetScreenFromRoot (display_info, ev->root);
@@ -1067,10 +1077,6 @@ handleDestroyNotify (DisplayInfo *display_info, XDestroyWindowEvent * ev)
         clientPassFocus (c->screen_info, c, c);
         clientUnframe (c, FALSE);
     }
-    else
-    {
-        compositorRemoveWindow (display_info, ev->window);
-    }
 }
 
 static void
@@ -1129,6 +1135,7 @@ handleMapNotify (DisplayInfo *display_info, XMapEvent * ev)
         {
             FLAG_UNSET (c->xfwm_flags, XFWM_FLAG_MAP_PENDING);
         }
+        compositorMapWindow (display_info, c->frame);
     }
     else if (myDisplayGetScreenFromRoot (display_info, ev->event))
     {
@@ -1177,6 +1184,7 @@ handleUnmapNotify (DisplayInfo *display_info, XUnmapEvent * ev)
 
         screen_info = c->screen_info;
         clientPassFocus (screen_info, c, c);
+        compositorUnmapWindow (display_info, c->frame);
         
         /*
          * ICCCM spec states that a client wishing to switch
@@ -1869,7 +1877,7 @@ handleClientMessage (DisplayInfo *display_info, XClientMessageEvent * ev)
                 getDesktopLayout(display_info, screen_info->xroot, screen_info->workspace_count, &screen_info->desktop_layout);
             }
         }
-        else if ((ev->message_type == display_info->atoms[NET_SYSTEM_TRAY_MANAGER]) && (ev->data.l[1] == screen_info->net_system_tray_selection) && (ev->format == 32))
+        else if ((ev->message_type == display_info->atoms[MANAGER]) && (ev->data.l[1] == screen_info->net_system_tray_selection) && (ev->format == 32))
         {
             TRACE ("root has received a net_system_tray_manager event");
             screen_info->systray = getSystrayWindow (display_info, screen_info->net_system_tray_selection);
