@@ -1,4 +1,4 @@
-/*      $Id: focus.c,v 1.40 2004/12/09 22:44:20 olivier Exp $
+/*      $Id: focus.c,v 1.41 2004/12/14 17:45:26 olivier Exp $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -561,7 +561,7 @@ clientGrabMouseButtonForAll (ScreenInfo *screen_info)
     {
         clientGrabMouseButton (c);
     }
-    clientClearLastGrab ();
+    clientClearLastUngrab ();
 }
 
 void
@@ -577,7 +577,7 @@ clientUngrabMouseButtonForAll (ScreenInfo *screen_info)
     {
         clientUngrabMouseButton (c);
     }
-    clientClearLastGrab ();
+    clientClearLastUngrab ();
 }
 
 void
@@ -611,13 +611,13 @@ clientPassGrabMouseButton (Client * c)
 }
 
 Client *
-clientGetLastGrab (void)
+clientGetLastUngrab (void)
 {
     return last_ungrab;
 }
 
 void
-clientClearLastGrab (void)
+clientClearLastUngrab (void)
 {
     last_ungrab = NULL;
 }
