@@ -1,4 +1,4 @@
-/*      $Id: workspaces.c,v 1.88 2004/11/06 13:04:03 olivier Exp $
+/*      $Id: workspaces.c,v 1.93 2004/12/10 05:09:24 olivier Exp $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -31,6 +31,7 @@
 #include <gtk/gtk.h>
 #include <libxfce4util/libxfce4util.h> 
 
+#include "display.h"
 #include "screen.h"
 #include "misc.h"
 #include "transients.h"
@@ -227,7 +228,7 @@ workspaceMove (ScreenInfo *screen_info, int rowmod, int colmod, Client * c2)
 void
 workspaceSwitch (ScreenInfo *screen_info, int new_ws, Client * c2)
 {
-    gboolean grab = (c2 == NULL);
+    DisplayInfo *display_info = screen_info->display_info;
     Client *c, *new_focus = NULL;
     Client *previous;
     GList *index;
@@ -265,13 +266,10 @@ workspaceSwitch (ScreenInfo *screen_info, int new_ws, Client * c2)
         return;
     }
 
-    if (grab)
-    {
-        /* Grab the pointer to avoid side effects with EnterNotify events */
-        XGrabPointer (myScreenGetXDisplay (screen_info), screen_info->gnome_win, 
-                      FALSE, EnterWindowMask, GrabModeAsync,
-                      GrabModeAsync, None, None, GDK_CURRENT_TIME);
-    }
+    /* Grab the pointer to avoid side effects with EnterNotify events */
+    XGrabPointer (myScreenGetXDisplay (screen_info), screen_info->gnome_win, 
+                  FALSE, EnterWindowMask, GrabModeAsync,
+                  GrabModeAsync, None, None, CurrentTime);
     
     screen_info->previous_ws = screen_info->current_ws;
     screen_info->current_ws = new_ws;
@@ -294,7 +292,7 @@ workspaceSwitch (ScreenInfo *screen_info, int new_ws, Client * c2)
             if (c == previous)
             {
                 FLAG_SET (previous->xfwm_flags, XFWM_FLAG_FOCUS);
-                clientSetFocus (screen_info, NULL, GDK_CURRENT_TIME, FOCUS_IGNORE_MODAL);
+                clientSetFocus (screen_info, NULL, myDisplayGetCurrentTime (display_info), FOCUS_IGNORE_MODAL);
             }
             if (!clientIsValidTransientOrModal (c))
             {
@@ -348,17 +346,14 @@ workspaceSwitch (ScreenInfo *screen_info, int new_ws, Client * c2)
         }
     }
 
-    setHint (myScreenGetXDisplay (screen_info), screen_info->xroot, win_workspace, new_ws);
+    setHint (display_info, screen_info->xroot, WIN_WORKSPACE, new_ws);
     data[0] = new_ws;
-    XChangeProperty (myScreenGetXDisplay (screen_info), screen_info->xroot, net_current_desktop, XA_CARDINAL, 32,
-        PropModeReplace, (unsigned char *) data, 1);
-    workspaceUpdateArea (screen_info);
+    XChangeProperty (myScreenGetXDisplay (screen_info), screen_info->xroot, 
+                     display_info->atoms[NET_CURRENT_DESKTOP], XA_CARDINAL, 32,
+                     PropModeReplace, (unsigned char *) data, 1);
     
     /* Ungrab the pointer we grabbed before mapping/unmapping all windows */
-    if (grab)
-    {
-        XUngrabPointer (myScreenGetXDisplay (screen_info), GDK_CURRENT_TIME);
-    }
+    XUngrabPointer (myScreenGetXDisplay (screen_info), CurrentTime);
     
     if (!(screen_info->params->click_to_focus))
     {
@@ -373,7 +368,7 @@ workspaceSwitch (ScreenInfo *screen_info, int new_ws, Client * c2)
     }
     if (new_focus)
     {
-        clientSetFocus (new_focus->screen_info, new_focus, GDK_CURRENT_TIME, NO_FOCUS_FLAG);
+        clientSetFocus (new_focus->screen_info, new_focus, myDisplayGetCurrentTime (display_info), NO_FOCUS_FLAG);
     }
 }
 
@@ -392,6 +387,7 @@ workspaceSetNames (ScreenInfo * screen_info, char *names, int length)
 void
 workspaceSetCount (ScreenInfo * screen_info, int count)
 {
+    DisplayInfo *display_info = NULL;
     Client *c;
     int i;
 
@@ -406,8 +402,9 @@ workspaceSetCount (ScreenInfo * screen_info, int count)
         return;
     }
 
-    setHint (myScreenGetXDisplay (screen_info), screen_info->xroot, win_workspace_count, count);
-    setHint (myScreenGetXDisplay (screen_info), screen_info->xroot, net_number_of_desktops, count);
+    display_info = screen_info->display_info;
+    setHint (display_info, screen_info->xroot, WIN_WORKSPACE_COUNT, count);
+    setHint (display_info, screen_info->xroot, NET_NUMBER_OF_DESKTOPS, count);
     screen_info->workspace_count = count;
 
     for (c = screen_info->clients, i = 0; i < screen_info->client_count; c = c->next, i++)
@@ -421,18 +418,19 @@ workspaceSetCount (ScreenInfo * screen_info, int count)
     {
         workspaceSwitch (screen_info, count - 1, NULL);
     }
-    setNetWorkarea (myScreenGetXDisplay (screen_info), screen_info->screen, screen_info->workspace_count, 
-                         gdk_screen_get_width (screen_info->gscr),
-                         gdk_screen_get_height (screen_info->gscr),
-                         screen_info->margins);
+    setNetWorkarea (display_info, screen_info->xroot, screen_info->workspace_count, 
+                    gdk_screen_get_width (screen_info->gscr),
+                    gdk_screen_get_height (screen_info->gscr),
+                    screen_info->margins);
     /* Recompute the layout based on the (changed) number of desktops */
-    getDesktopLayout(myScreenGetXDisplay (screen_info), screen_info->xroot, screen_info->workspace_count, 
-                         &screen_info->desktop_layout);
+    getDesktopLayout(display_info, screen_info->xroot, screen_info->workspace_count, 
+                     &screen_info->desktop_layout);
 }
 
 void
 workspaceUpdateArea (ScreenInfo *screen_info)
 {
+    DisplayInfo *display_info = NULL;
     Client *c;
     int prev_top;
     int prev_left;
@@ -446,6 +444,7 @@ workspaceUpdateArea (ScreenInfo *screen_info)
 
     TRACE ("entering workspaceUpdateArea");
 
+    display_info = screen_info->display_info;
     prev_top = screen_info->margins[TOP];
     prev_left = screen_info->margins[LEFT];
     prev_right = screen_info->margins[RIGHT];
@@ -473,9 +472,9 @@ workspaceUpdateArea (ScreenInfo *screen_info)
         || (prev_bottom != screen_info->margins[BOTTOM]))
     {
         TRACE ("Margins have changed, updating net_workarea");
-        setNetWorkarea (myScreenGetXDisplay (screen_info), screen_info->screen, screen_info->workspace_count, 
-                             gdk_screen_get_width (screen_info->gscr),
-                             gdk_screen_get_height (screen_info->gscr),
-                             screen_info->margins);
+        setNetWorkarea (display_info, screen_info->xroot, screen_info->workspace_count, 
+                        gdk_screen_get_width (screen_info->gscr),
+                        gdk_screen_get_height (screen_info->gscr),
+                        screen_info->margins);
     }
 }
